@@ -1,8 +1,8 @@
 use strict;
 use warnings;
 package AnyEvent::CurrentCost;
-BEGIN {
-  $AnyEvent::CurrentCost::VERSION = '1.112970';
+{
+  $AnyEvent::CurrentCost::VERSION = '1.130190';
 }
 
 # ABSTRACT: AnyEvent module for reading from Current Cost energy meters
@@ -12,6 +12,7 @@ use constant DEBUG => $ENV{ANYEVENT_CURRENT_COST_DEBUG};
 use base qw/Device::CurrentCost/;
 use AnyEvent;
 use AnyEvent::Handle;
+use AnyEvent::SerialPort;
 use Carp qw/croak carp/;
 use Sub::Name;
 
@@ -20,7 +21,6 @@ sub new {
   my ($pkg, %p) = @_;
   croak $pkg.q{->new: 'callback' parameter is required} unless ($p{callback});
   my $self = $pkg->SUPER::new(%p);
-  $self->_setup_handle($p{filehandle}) if (exists $p{filehandle});
   $self;
 }
 
@@ -43,34 +43,35 @@ sub _error {
 
 sub open {
   my $self = shift;
-  my $fh = $self->SUPER::open;
-  $self->_setup_handle($self->SUPER::open);
-}
-
-sub _setup_handle {
-  my ($self, $fh) = @_;
-  $self->{filehandle} = $fh;
-  my $handle; $handle =
-    AnyEvent::Handle->new(fh => $fh,
-                          on_error => (subname 'on_error' => sub {
-                            my ($handle, $fatal, $msg) = @_;
-                            print STDERR $handle.": error $msg\n" if DEBUG;
-                            $handle->destroy;
-                            $self->_error($fatal, 'Error: '.$msg);
-                          }),
-                          on_rtimeout => (subname 'on_rtimeout' => sub {
-                            my $rbuf = \$handle->{rbuf};
-                            carp $handle, ": Discarding '", $$rbuf, "'\n";
-                            $$rbuf = '';
-                            $handle->rtimeout(undef);
-                          }),
-                         );
+  my $fh = $self->filehandle;
+  my $handle =
+    $fh
+      ? AnyEvent::Handle->new(fh => $fh)
+        : AnyEvent::SerialPort->new(serial_port =>
+                                    [ $self->device,
+                                      [ baudrate => $self->baud ] ]);
+  print STDERR ref $self, "->open: created ", $handle, "\n" if DEBUG;
   $self->{handle} = $handle;
-  $handle->push_read(ref $self => $self,
-                     subname 'push_read_cb' => sub {
-                       $self->{callback}->(@_);
-                       return;
-                     });
+  $handle->on_error(subname 'on_error' => sub {
+                      my ($handle, $fatal, $msg) = @_;
+                      print STDERR $handle.": error $msg\n" if DEBUG;
+                      $handle->destroy;
+                      $self->_error($fatal, 'Error: '.$msg);
+                    });
+  $handle->on_rtimeout(subname 'on_rtimeout' => sub {
+                         my $rbuf = \$handle->{rbuf};
+                         carp $handle, ": Discarding '", $$rbuf, "'\n";
+                         $$rbuf = '';
+                         $handle->rtimeout(undef);
+                       });
+  $handle->on_read(subname 'on_read_cb' => sub {
+                     my ($hdl) = @_;
+                     $hdl->push_read(ref $self => $self,
+                                     subname 'push_read_cb' => sub {
+                                       $self->{callback}->(@_);
+                                       1;
+                                     });
+                   });
 }
 
 sub _time_now {
@@ -88,7 +89,7 @@ sub anyevent_read_type {
       return unless ($res);
       print STDERR "After: ", (unpack 'H*', $$rbuf), "\n" if DEBUG;
       $handle->rtimeout($self->{discard_timeout}) if ($$rbuf && length $$rbuf);
-      $res = $cb->($res) and return $res;
+      $res = $cb->($res);
     }
   }
 }
@@ -104,7 +105,7 @@ AnyEvent::CurrentCost - AnyEvent module for reading from Current Cost energy met
 
 =head1 VERSION
 
-version 1.112970
+version 1.130190
 
 =head1 SYNOPSIS
 
